# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .project_entity_response import ProjectEntityResponse
from ...base_types.types.qualified_count import QualifiedCount
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class ProjectEntitiesResponse(UniversalBaseModel):
    """
    Examples
    --------
    from sayari.base_types import QualifiedCount
    from sayari.project_entity import (
        Address,
        AttributeValues,
        BusinessPurpose,
        ProjectEntitiesResponse,
        ProjectEntityMatchExplanation,
        ProjectEntityMatchResponse,
        ProjectEntityResponse,
        ProjectRiskCategory,
        ProjectRiskFactor,
        SourceField,
        TradeCounts,
        UpstreamInfo,
    )

    ProjectEntitiesResponse(
        limit=20,
        data=[
            ProjectEntityResponse(
                project_entity_id="52z4Wa",
                project_id="YVB88Y",
                label="VTB Bank",
                upload_ids=[],
                strength="strong",
                created_at="2025-04-22 22:54:00.913586+00",
                attributes={
                    "name": AttributeValues(
                        match_resolution=True,
                        values=["VTB Bank"],
                    ),
                    "country": AttributeValues(
                        match_resolution=True,
                        values=["RUS"],
                    ),
                    "address": AttributeValues(
                        match_resolution=True,
                        values=["Moscow"],
                    ),
                    "identifier": AttributeValues(
                        match_resolution=True,
                        values=["253400V1H6ART1UQ0N98"],
                    ),
                },
                countries=[
                    "USA",
                    "CYP",
                    "ITA",
                    "CHN",
                    "RUS",
                    "UKR",
                    "IND",
                    "IRL",
                    "KAZ",
                    "DEU",
                ],
                risk_categories=[
                    ProjectRiskCategory(
                        id="regulatory_action",
                        label="Regulatory action",
                        risk_factors=["regulatory_action"],
                    ),
                    ProjectRiskCategory(
                        id="export_controls",
                        label="Export controls",
                        risk_factors=["meu_list_contractors", "export_controls"],
                    ),
                    ProjectRiskCategory(
                        id="political_exposure",
                        label="Political exposure",
                        risk_factors=["soe_adjacent", "pep_adjacent"],
                    ),
                    ProjectRiskCategory(
                        id="sanctions",
                        label="Sanctions",
                        risk_factors=["sanctioned", "formerly_sanctioned"],
                    ),
                    ProjectRiskCategory(
                        id="adverse_media",
                        label="Adverse media",
                        risk_factors=["reputational_risk_other"],
                    ),
                ],
                risk_factors=[
                    ProjectRiskFactor(
                        id="regulatory_action",
                    ),
                    ProjectRiskFactor(
                        id="sanctioned",
                    ),
                    ProjectRiskFactor(
                        id="meu_list_contractors",
                    ),
                    ProjectRiskFactor(
                        id="reputational_risk_other",
                    ),
                    ProjectRiskFactor(
                        id="pep_adjacent",
                    ),
                    ProjectRiskFactor(
                        id="state_owned",
                    ),
                    ProjectRiskFactor(
                        id="export_controls",
                    ),
                    ProjectRiskFactor(
                        id="formerly_sanctioned",
                    ),
                    ProjectRiskFactor(
                        id="basel_aml",
                    ),
                    ProjectRiskFactor(
                        id="cpi_score",
                    ),
                ],
                upstream=UpstreamInfo(
                    risk_factors=[],
                    countries=[],
                    trade_counts=TradeCounts(
                        shipper_of=56,
                        receiver_of=2,
                    ),
                    has_upstream=False,
                    products=[],
                ),
                tags=[],
                case="not_assigned",
                matches=[
                    ProjectEntityMatchResponse(
                        match_id="52z4Wa:dy-rh2g0QtzUN_jC_e9S_A",
                        sayari_entity_id="dy-rh2g0QtzUN_jC_e9S_A",
                        type="company",
                        label='ОТКРЫТОЕ АКЦИОНЕРНОЕ ОБЩЕСТВО "РОССИЙСКИЕ ЖЕЛЕЗНЫЕ ДОРОГИ"',
                        match_explanation=[
                            ProjectEntityMatchExplanation(
                                field="name",
                                quality="high",
                                matches=[
                                    "<em>БАНК</em> <em>ВТБ</em> (ПАО)",
                                    "<em>VTB</em> <em>BANK</em> (PJSC)",
                                    "<em>БАНК</em> <em>ВТБ</em> (ПУБЛИЧНОЕ АКЦИОНЕРНОЕ ОБЩЕСТВО)",
                                    "<em>VTB</em> <em>BANK</em> (PUBLIC JOINT STOCK COMPANY)",
                                    "<em>VTB</em> <em>Bank</em>",
                                ],
                            ),
                            ProjectEntityMatchExplanation(
                                field="address",
                                quality="medium",
                                matches=[
                                    "109147 <em>Moscow</em>, st. Vorontsovskaya, 43 building 1",
                                    "119121 <em>Moscow</em>, st. Plyushchikha, 37.",
                                    "Vorontsovskaya Str., 43 <em>Moscow</em> 109044 RUSSIAN FEDERATION",
                                    "37 Plyushchikha ul., <em>Moscow</em>, 119121, Russia",
                                    "Bashnya Zapad, Kompleks Federatsiya, 12, nab. Presnenskaya, <em>Moscow</em>, 123317, Russia",
                                ],
                            ),
                            ProjectEntityMatchExplanation(
                                field="country",
                                quality="high",
                                matches=["<em>RUS</em>"],
                            ),
                            ProjectEntityMatchExplanation(
                                field="identifier",
                                quality="high",
                                matches=["<em>253400V1H6ART1UQ0N98</em>"],
                            ),
                        ],
                        countries=[
                            "USA",
                            "CYP",
                            "ITA",
                            "CHN",
                            "RUS",
                            "UKR",
                            "IND",
                            "IRL",
                            "KAZ",
                            "DEU",
                        ],
                        risk_categories=[
                            ProjectRiskCategory(
                                id="regulatory_action",
                                label="Regulatory action",
                                risk_factors=["regulatory_action"],
                            ),
                            ProjectRiskCategory(
                                id="export_controls",
                                label="Export controls",
                                risk_factors=[
                                    "meu_list_contractors",
                                    "export_controls",
                                ],
                            ),
                            ProjectRiskCategory(
                                id="political_exposure",
                                label="Political exposure",
                                risk_factors=["soe_adjacent", "pep_adjacent"],
                            ),
                            ProjectRiskCategory(
                                id="sanctions",
                                label="Sanctions",
                                risk_factors=["sanctioned", "formerly_sanctioned"],
                            ),
                            ProjectRiskCategory(
                                id="adverse_media",
                                label="Adverse media",
                                risk_factors=["reputational_risk_other"],
                            ),
                        ],
                        risk_factors=[
                            ProjectRiskFactor(
                                id="regulatory_action",
                            ),
                            ProjectRiskFactor(
                                id="sanctioned",
                            ),
                            ProjectRiskFactor(
                                id="meu_list_contractors",
                            ),
                            ProjectRiskFactor(
                                id="reputational_risk_other",
                            ),
                            ProjectRiskFactor(
                                id="pep_adjacent",
                            ),
                            ProjectRiskFactor(
                                id="state_owned",
                            ),
                            ProjectRiskFactor(
                                id="export_controls",
                            ),
                            ProjectRiskFactor(
                                id="formerly_sanctioned",
                            ),
                            ProjectRiskFactor(
                                id="basel_aml",
                            ),
                            ProjectRiskFactor(
                                id="cpi_score",
                            ),
                        ],
                        business_purpose=[
                            BusinessPurpose(
                                standard="ISIC4",
                                code="6419",
                            ),
                            BusinessPurpose(
                                code="64.19",
                                standard="unknown",
                            ),
                            BusinessPurpose(
                                code="64.9",
                                standard="unknown",
                            ),
                            BusinessPurpose(
                                code="62.09",
                                standard="unknown",
                            ),
                            BusinessPurpose(
                                standard="NACE2",
                                code="64.19",
                            ),
                            BusinessPurpose(
                                code="64.19",
                                standard="unknown",
                            ),
                            BusinessPurpose(
                                standard="ISIC4",
                                code="649",
                            ),
                            BusinessPurpose(
                                code="66.12",
                                standard="unknown",
                            ),
                            BusinessPurpose(
                                standard="ISIC4",
                                code="6419",
                            ),
                            BusinessPurpose(
                                standard="ISIC4",
                                code="6209",
                            ),
                            BusinessPurpose(
                                standard="ISIC4",
                                code="6612",
                            ),
                        ],
                        upstream=UpstreamInfo(
                            risk_factors=[],
                            countries=[],
                            trade_counts=TradeCounts(
                                shipper_of=56,
                                receiver_of=2,
                            ),
                            has_upstream=False,
                            products=[],
                        ),
                        sources=[
                            SourceField(
                                id="eb361ba33d05a9d15b2d28aea739362a",
                                label="USA SEC Central Index Key Database",
                            ),
                            SourceField(
                                id="a8c6ee1cd4dfc952105ee8c0e4836f08",
                                label="Acuris Risk Intelligence KYC6 (3rd Party Data)",
                            ),
                            SourceField(
                                id="3322a755c28851ec06eda3280cc2a2b2",
                                label="Russia ClearSpending (3rd Party Data)",
                            ),
                            SourceField(
                                id="93de05b55a68cc964f34bbfc7ec7bc86",
                                label="Consolidated Canadian Autonomous Sanctions List",
                            ),
                            SourceField(
                                id="fdfedf11a5122e2aef9dc3f176c7427d",
                                label="Sayari Intel (Sayari Curated Risk Data) - Archived",
                            ),
                            SourceField(
                                id="e517c714b560f159a89a9501ed7dd84e",
                                label="Infocredit Cyprus Company Shareholders Database (3rd Party Data)",
                            ),
                            SourceField(
                                id="bef2f30139ea37c723ecb0f953b5f024",
                                label="Australia Consolidated Sanctions List",
                            ),
                            SourceField(
                                id="9048153c770d941afe9500c6ad810a80",
                                label="Kazakhstan DFO Registry",
                            ),
                            SourceField(
                                id="635021e4e01979cdf95fa7ee76501696",
                                label="Russia E-Disclosures",
                            ),
                            SourceField(
                                id="e85d865943ee6d8369307569d2ad9de0",
                                label="Acuris Risk Intelligence Adverse Media Data",
                            ),
                            SourceField(
                                id="4ff85397f508e4ff8e71c5ca5ea895dd",
                                label="Japan Ministry of Finance Economic Sanctions List",
                            ),
                            SourceField(
                                id="ce873f184abee5b13c9e89791bd78229",
                                label="Ukraine Significant Shareholders of Securities Issuers",
                            ),
                            SourceField(
                                id="8511f57f6098dcc36e3a814c3ea5bdd7",
                                label="Uzbekistan Imports & Exports (January 2022 - Present)",
                            ),
                            SourceField(
                                id="e6e4acebdea4e9e71af8d3586d34fcdd",
                                label="Ukraine Sanctions List",
                            ),
                            SourceField(
                                id="db3416894cd5f0c4d2d6ecc79bdaf366",
                                label="EDI Publicly-Listed Global Security Issuers (3rd Party Data)",
                            ),
                            SourceField(
                                id="b812677c0a32a1746b3ac741c7b97ae0",
                                label="Legal Entity Identifier (LEI) Registry (3rd Party Data)",
                            ),
                            SourceField(
                                id="a9a18eeb901e4376c912e95dc05ceb78",
                                label="Russia Federal Tax Registry",
                            ),
                            SourceField(
                                id="d357238101a2bb02d97d5d77a56d9455",
                                label="New Zealand Russia Sanctions",
                            ),
                            SourceField(
                                id="f4ce10ac8bdc0bd9bcb31cb868b4b539",
                                label="EU Financial Sanctions List",
                            ),
                            SourceField(
                                id="c10d482320f207d92aa814519c3bd686",
                                label="UK Consolidated Sanctions List",
                            ),
                            SourceField(
                                id="d1bce737c158efddbef5048d63aaf124",
                                label="Ukraine Ministry of Justice Registry of Companies",
                            ),
                            SourceField(
                                id="92edb8fe6615498f6e3e7b0e220f74e6",
                                label="USA Treasury OFAC Specially Designated Nationals (SDN) List",
                            ),
                            SourceField(
                                id="66dfefb726ae00fde8f09f34c5578d35",
                                label="Russia Imports & Exports (January 2023 - Present)",
                            ),
                            SourceField(
                                id="ad6dc774c74b92159422deabc5aee661",
                                label="France National Asset Freeze Register",
                            ),
                            SourceField(
                                id="b97ad5c858093e5227b2359a0078696f",
                                label="USA SAM.gov Entity Exclusions Database",
                            ),
                            SourceField(
                                id="ddbf93a5c5d568ccdb0c2455f7ecbfc8",
                                label="USA CorpWatch SEC 10-K Exhibit 21 Database (3rd Party Data)",
                            ),
                            SourceField(
                                id="52a7bc4a30e787544ad2d95e7f54adca",
                                label="USA OFAC Non-SDN Consolidated Sanctions List",
                            ),
                            SourceField(
                                id="f43658cf2a19452fb45b830edeeeea44",
                                label="Russia Central Bank Financial Institution UBO Disclosures",
                            ),
                            SourceField(
                                id="828f81c7c31410de845d9e6ed1a0f3e6",
                                label="Russia Central Bank Financial Institutions",
                            ),
                            SourceField(
                                id="56bce0e008204712e302271ddd7b4fb1",
                                label="Russia Federal Tax Registry (2018)",
                            ),
                            SourceField(
                                id="e0b32f41da3b4e6c75f1f7bf6fe1c9c5",
                                label="S&P Global Market Intelligence Database (3rd Party Data)",
                            ),
                            SourceField(
                                id="10a889609ddae6cc31773feebe48166b",
                                label="Switzerland SECO Sanctions List",
                            ),
                            SourceField(
                                id="ded706ef28cdc649b6fc5142bc542890",
                                label="EU Sanctions Map",
                            ),
                            SourceField(
                                id="417400a1ac439843fda5bb2f05fb4161",
                                label="USA Consolidated Screening List",
                            ),
                            SourceField(
                                id="765d0dc816bb7db35031c43a37290350",
                                label="Ireland Companies Registration Office Database",
                            ),
                        ],
                        addresses=[
                            Address(
                                value="190000, г Санкт-Петербург, ул Большая Морская, дом 29",
                            ),
                            Address(
                                value="109147 г. Москва, ул. Воронцовская, д.43 стр.1",
                            ),
                            Address(
                                value="119121 г. Москва, ул. Плющиха, д. 37.",
                            ),
                            Address(
                                value="43, Vorontsovskaya str., Moscow, 109044, Russia",
                            ),
                            Address(
                                value="11, lit A, Degtyarnyy pereulok, St. Petersburg, 191144, Russia",
                            ),
                            Address(
                                value="29, Bolshaya Morskaya str., St. Petersburg, 190000, Russia",
                            ),
                        ],
                        hs_codes=[
                            "847150",
                            "8471500000",
                            "852351",
                            "8523519101",
                            "860610",
                            "8606100000",
                            "271019",
                            "2710196601",
                            "490700",
                            "4907003000",
                            "711890",
                            "7118900000",
                            "490700",
                            "4907003000",
                            "711890",
                            "7118900000",
                        ],
                        created_at="2025-04-22 22:54:00.913586+00",
                        updated_at="2025-04-22 22:54:00.913586+00",
                    )
                ],
            )
        ],
        size=QualifiedCount(
            count=1,
            qualifier="eq",
        ),
    )
    """

    limit: int
    data: typing.List[ProjectEntityResponse]
    size: QualifiedCount
    next_token: typing.Optional[str] = None
    prev_token: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

MonetaryValueContext = typing.Union[
    typing.Literal[
        "assessed_value",
        "cadastral_value",
        "carriage_and_insurance_paid_to",
        "carriage_paid_to",
        "contract_value",
        "cost_and_freight",
        "cost_insurance_and_freight",
        "delivered_at_place",
        "delivered_at_place_unloaded",
        "delivered_duty_paid",
        "ex_works",
        "free_alongside_ship",
        "free_carrier",
        "free_on_board",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing
import pydantic


class UpdateProjectEntityAttributeRequest(UniversalBaseModel):
    """
    Examples
    --------
    from sayari.project_entity_attributes import UpdateProjectEntityAttributeRequest

    UpdateProjectEntityAttributeRequest(
        field="name",
        value="updated name",
        match_resolution=True,
    )
    """

    field: str
    value: str
    match_resolution: bool

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class AdditionalInformationProperties(UniversalBaseModel):
    date: typing.Optional[str] = pydantic.Field(default=None)
    """
    as-of date of attribute
    """

    extra: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    extra information of attribute
    """

    from_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    start date of attribute
    """

    to_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    end date of attribute
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of additional information being conveyed
    """

    value: typing.Optional[str] = pydantic.Field(default=None)
    """
    The additional information itself
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from ...shared_types.types.match_strength_enum import MatchStrengthEnum
from .attribute_values import AttributeValues
from .project_risk_category import ProjectRiskCategory
from .project_risk_factor import ProjectRiskFactor
from .upstream_info import UpstreamInfo
from .tag_response import TagResponse
from ...shared_types.types.case_status import CaseStatus
from .project_entity_match_response import ProjectEntityMatchResponse
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class ProjectEntityResponse(UniversalBaseModel):
    project_entity_id: str
    project_id: str
    label: str
    upload_ids: typing.List[str]
    strength: MatchStrengthEnum
    created_at: str
    attributes: typing.Dict[str, AttributeValues]
    countries: typing.List[str]
    risk_categories: typing.List[ProjectRiskCategory]
    risk_factors: typing.List[ProjectRiskFactor]
    upstream: UpstreamInfo
    tags: typing.List[TagResponse]
    case: typing.Optional[CaseStatus] = None
    matches: typing.List[ProjectEntityMatchResponse]
    updated_at: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .project_risk_category import ProjectRiskCategory
from .project_risk_factor import ProjectRiskFactor
from .business_purpose import BusinessPurpose
from .upstream_info import UpstreamInfo
from .source_field import SourceField
from .address import Address
from .match_profile_enum import MatchProfileEnum
from .project_entity_match_explanation import ProjectEntityMatchExplanation
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class ProjectEntityMatchResponse(UniversalBaseModel):
    match_id: str
    sayari_entity_id: str
    type: str
    label: str
    countries: typing.List[str]
    risk_categories: typing.List[ProjectRiskCategory]
    risk_factors: typing.List[ProjectRiskFactor]
    business_purpose: typing.List[BusinessPurpose]
    upstream: UpstreamInfo
    sources: typing.List[SourceField]
    addresses: typing.List[Address]
    hs_codes: typing.List[str]
    created_at: str
    updated_at: typing.Optional[str] = None
    match_profile: typing.Optional[MatchProfileEnum] = None
    deleted_at: typing.Optional[str] = None
    match_explanation: typing.Optional[typing.List[ProjectEntityMatchExplanation]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .update_project_entity_attribute_response_data import UpdateProjectEntityAttributeResponseData
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing
import pydantic


class UpdateProjectEntityAttributeResponse(UniversalBaseModel):
    """
    Examples
    --------
    from sayari.project_entity_attributes import (
        UpdateProjectEntityAttributeResponse,
        UpdateProjectEntityAttributeResponseData,
    )

    UpdateProjectEntityAttributeResponse(
        data=UpdateProjectEntityAttributeResponseData(
            id="xG8wYP",
            field="name",
            value="updated name",
            match_resolution=True,
        ),
    )
    """

    data: UpdateProjectEntityAttributeResponseData

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .field_match_quality import FieldMatchQuality
import pydantic
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ProjectEntityMatchExplanation(UniversalBaseModel):
    field: str
    quality: FieldMatchQuality = pydantic.Field()
    """
    Quality of the match
    """

    description: typing.Optional[typing.List[str]] = None
    matches: typing.Optional[typing.List[str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .types.create_resolved_project_entity_request import CreateResolvedProjectEntityRequest
from ..core.request_options import RequestOptions
from .types.single_project_entity_response import SingleProjectEntityResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.pydantic_utilities import parse_obj_as
from ..shared_errors.errors.bad_request import BadRequest
from ..shared_errors.types.bad_request_response import BadRequestResponse
from ..shared_errors.errors.unauthorized import Unauthorized
from ..shared_errors.types.unauthorized_response import UnauthorizedResponse
from ..shared_errors.errors.not_found import NotFound
from ..shared_errors.types.not_found_response import NotFoundResponse
from ..shared_errors.errors.method_not_allowed import MethodNotAllowed
from ..shared_errors.types.method_not_allowed_response import MethodNotAllowedResponse
from ..shared_errors.errors.rate_limit_exceeded import RateLimitExceeded
from ..shared_errors.types.rate_limit_response import RateLimitResponse
from ..shared_errors.errors.internal_server_error import InternalServerError
from ..shared_errors.types.internal_server_error_response import InternalServerErrorResponse
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from .types.project_entities_filter import ProjectEntitiesFilter
from .types.project_entities_response import ProjectEntitiesResponse
from .types.resolution_attributes import ResolutionAttributes
from .types.project_entity_id_response import ProjectEntityIdResponse
from .types.save_project_entity_body import SaveProjectEntityBody
from .types.sort_order import SortOrder
from .types.project_risk_changes_response import ProjectRiskChangesResponse
from ..shared_errors.errors.unprocessable_content import UnprocessableContent
from ..shared_errors.types.unprocessable_content_response import UnprocessableContentResponse
from .types.project_entity_risk_changes_response import ProjectEntityRiskChangesResponse
from ..generated_types.types.risk import Risk
from ..generated_types.types.country import Country
from ..supply_chain.types.upstream_trade_traversal_response import UpstreamTradeTraversalResponse
from .types.project_entity_risk_summary_filters import ProjectEntityRiskSummaryFilters
from .types.project_entity_risk_summary_response import ProjectEntityRiskSummaryResponse
from .types.project_entity_supply_chain_summary_response import ProjectEntitySupplyChainSummaryResponse
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ProjectEntityClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create_project_entity(
        self,
        project_id: str,
        *,
        request: CreateResolvedProjectEntityRequest,
        enable_llm_clean: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SingleProjectEntityResponse:
        """
        The resolution endpoints allow users to search for matching entities against a provided list of attributes. The endpoint is similar to the search endpoint, except it's tuned to only return the best match so the client doesn't need to do as much or any post-processing work to filter down results.

        Parameters
        ----------
        project_id : str

        request : CreateResolvedProjectEntityRequest

        enable_llm_clean : typing.Optional[bool]
            Whether to enable LLM-based data cleaning to remove noise and standardize entity attributes. Defaults to true if not supplied. Set to false to disable LLM cleaning.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SingleProjectEntityResponse

        Examples
        --------
        from sayari import Sayari
        from sayari.project_entity import CreateResolvedProjectEntityRequest

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.project_entity.create_project_entity(
            project_id="YVB88Y",
            request=CreateResolvedProjectEntityRequest(
                name=["VTB Bank"],
                country=["RUS"],
                address=["Moscow"],
                identifier=["253400V1H6ART1UQ0N98"],
                profile="corporate",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/create",
            method="POST",
            params={
                "enable_llm_clean": enable_llm_clean,
            },
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=CreateResolvedProjectEntityRequest, direction="write"
            ),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    SingleProjectEntityResponse,
                    parse_obj_as(
                        type_=SingleProjectEntityResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_project_entities(
        self,
        project_id: str,
        *,
        next: typing.Optional[str] = None,
        prev: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        filter: typing.Optional[ProjectEntitiesFilter] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectEntitiesResponse:
        """
        Retrieves a list of entities for a specific project with pagination support.

        **Response Formats:**
        - **JSON** (default): Returns structured data with nested objects
        - **CSV**: Returns tabular data with dynamic columns for attributes and risk categories

        **CSV Format:**
        The CSV response includes dynamic columns based on the data:
        - `attribute_{field_name}`: Dynamic columns for each attribute field found in the data
        - `risk_category_{category_id}`: Dynamic columns for each risk category found in the data
        - Standard columns: project_id, project_entity_id, label, project_entity_url, upload_ids, strength, countries, tags, case_status, created_at, match_count, upstream_products, upstream_risk_factors, upstream_countries

        Use the `Accept: text/csv` header to request CSV format.

        Parameters
        ----------
        project_id : str

        next : typing.Optional[str]
            The pagination token for the next page of projects.

        prev : typing.Optional[str]
            The pagination token for the previous page of projects.

        limit : typing.Optional[int]
            Limit total values returned for projects. Defaults to 100. Max 100.

        filter : typing.Optional[ProjectEntitiesFilter]
            Filter the project entities. Supports both dot notation (e.g., 'filter.attribute.name') and bracket notation (e.g., 'filter[attribute][name]') for nested field filtering.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectEntitiesResponse

        Examples
        --------
        from sayari import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.project_entity.get_project_entities(
            project_id="YVB88Y",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities",
            method="GET",
            params={
                "next": next,
                "prev": prev,
                "limit": limit,
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=ProjectEntitiesFilter, direction="write"
                ),
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ProjectEntitiesResponse,
                    parse_obj_as(
                        type_=ProjectEntitiesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_project_entity(
        self, project_id: str, project_entity_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SingleProjectEntityResponse:
        """
        Retrieves a specific entity in a project.

        Parameters
        ----------
        project_id : str

        project_entity_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SingleProjectEntityResponse

        Examples
        --------
        from sayari import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.project_entity.get_project_entity(
            project_id="project_id",
            project_entity_id="project_entity_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/{jsonable_encoder(project_entity_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    SingleProjectEntityResponse,
                    parse_obj_as(
                        type_=SingleProjectEntityResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_project_entity(
        self, project_id: str, project_entity_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Deletes a project entity.

        Parameters
        ----------
        project_id : str

        project_entity_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from sayari import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.project_entity.delete_project_entity(
            project_id="project_id",
            project_entity_id="project_entity_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/{jsonable_encoder(project_entity_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_project_entity_match(
        self,
        project_id: str,
        project_entity_id: str,
        match_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes a specific project entity match.

        Parameters
        ----------
        project_id : str

        project_entity_id : str

        match_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from sayari import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.project_entity.delete_project_entity_match(
            project_id="project_id",
            project_entity_id="project_entity_id",
            match_id="match_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/{jsonable_encoder(project_entity_id)}/matches/{jsonable_encoder(match_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def project_entity_exists(
        self, project_id: str, *, request: ResolutionAttributes, request_options: typing.Optional[RequestOptions] = None
    ) -> ProjectEntityIdResponse:
        """
        Checks if a project entity with the given attributes already exists.

        Parameters
        ----------
        project_id : str

        request : ResolutionAttributes

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectEntityIdResponse

        Examples
        --------
        from sayari import Sayari
        from sayari.project_entity import ResolutionAttributes

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.project_entity.project_entity_exists(
            project_id="YVB88Y",
            request=ResolutionAttributes(
                name=["Example Company"],
                country=["USA"],
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/exists",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=ResolutionAttributes, direction="write"
            ),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ProjectEntityIdResponse,
                    parse_obj_as(
                        type_=ProjectEntityIdResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def save_project_entity(
        self,
        project_id: str,
        *,
        request: SaveProjectEntityBody,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectEntityIdResponse:
        """
        Stores a project entity given entity IDs and a list of attributes.

        Parameters
        ----------
        project_id : str

        request : SaveProjectEntityBody

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectEntityIdResponse

        Examples
        --------
        from sayari import Sayari
        from sayari.project_entity import ResolutionAttributes, SaveProjectEntityBody

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.project_entity.save_project_entity(
            project_id="YVB88Y",
            request=SaveProjectEntityBody(
                entity_ids=["entity_id_1", "entity_id_2"],
                attributes=ResolutionAttributes(
                    name=["Example Company"],
                    country=["USA"],
                ),
                profile="corporate",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/save",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=SaveProjectEntityBody, direction="write"
            ),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ProjectEntityIdResponse,
                    parse_obj_as(
                        type_=ProjectEntityIdResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_project_risk_changes(
        self,
        project_id: str,
        *,
        limit: typing.Optional[int] = None,
        next: typing.Optional[str] = None,
        prev: typing.Optional[str] = None,
        sort_date: typing.Optional[SortOrder] = None,
        filter_from: typing.Optional[str] = None,
        filter_to: typing.Optional[str] = None,
        filter_risk_factor: typing.Optional[typing.Sequence[str]] = None,
        filter_risk_category: typing.Optional[typing.Sequence[str]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectRiskChangesResponse:
        """
        Retrieves the risk changes for all project entities in a project.

        Parameters
        ----------
        project_id : str

        limit : typing.Optional[int]
            The maximum number of risk changes to return. Default is 100.

        next : typing.Optional[str]
            Token to retrieve the next page of risk changes

        prev : typing.Optional[str]
            Token to retrieve the previous page of risk changes

        sort_date : typing.Optional[SortOrder]
            Sort by timestamp in ascending or descending order. Default is desc.

        filter_from : typing.Optional[str]
            Filter risk changes from the provided date

        filter_to : typing.Optional[str]
            Filter risk changes up to the provided date

        filter_risk_factor : typing.Optional[typing.Sequence[str]]
            Filter risk changes for the provided risk factors

        filter_risk_category : typing.Optional[typing.Sequence[str]]
            Filter risk changes for risk factors from the provided risk categories

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectRiskChangesResponse

        Examples
        --------
        from sayari import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.project_entity.get_project_risk_changes(
            project_id="YVB88Y",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/changes/risk",
            method="GET",
            params={
                "limit": limit,
                "next": next,
                "prev": prev,
                "sort.date": sort_date,
                "filter.from": filter_from,
                "filter.to": filter_to,
                "filter.risk_factor": filter_risk_factor,
                "filter.risk_category": filter_risk_category,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ProjectRiskChangesResponse,
                    parse_obj_as(
                        type_=ProjectRiskChangesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableContent(
                    typing.cast(
                        UnprocessableContentResponse,
                        parse_obj_as(
                            type_=UnprocessableContentResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_project_entity_risk_changes(
        self,
        project_id: str,
        project_entity_id: str,
        *,
        limit: typing.Optional[int] = None,
        next: typing.Optional[str] = None,
        prev: typing.Optional[str] = None,
        sort_date: typing.Optional[SortOrder] = None,
        filter_from: typing.Optional[str] = None,
        filter_to: typing.Optional[str] = None,
        filter_risk_factor: typing.Optional[typing.Sequence[str]] = None,
        filter_risk_category: typing.Optional[typing.Sequence[str]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectEntityRiskChangesResponse:
        """
        Retrieves the risk changes for a single project entity.

        Parameters
        ----------
        project_id : str

        project_entity_id : str

        limit : typing.Optional[int]
            The maximum number of risk changes to return. Default is 100.

        next : typing.Optional[str]
            Token to retrieve the next page of risk changes

        prev : typing.Optional[str]
            Token to retrieve the previous page of risk changes

        sort_date : typing.Optional[SortOrder]
            Sort by timestamp in ascending or descending order. Default is desc.

        filter_from : typing.Optional[str]
            Filter risk changes from the provided date

        filter_to : typing.Optional[str]
            Filter risk changes up to the provided date

        filter_risk_factor : typing.Optional[typing.Sequence[str]]
            Filter risk changes for the provided risk factors

        filter_risk_category : typing.Optional[typing.Sequence[str]]
            Filter risk changes for risk factors from the provided risk categories

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectEntityRiskChangesResponse

        Examples
        --------
        from sayari import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.project_entity.get_project_entity_risk_changes(
            project_id="YVB88Y",
            project_entity_id="52z4Wa",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/{jsonable_encoder(project_entity_id)}/changes/risk",
            method="GET",
            params={
                "limit": limit,
                "next": next,
                "prev": prev,
                "sort.date": sort_date,
                "filter.from": filter_from,
                "filter.to": filter_to,
                "filter.risk_factor": filter_risk_factor,
                "filter.risk_category": filter_risk_category,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ProjectEntityRiskChangesResponse,
                    parse_obj_as(
                        type_=ProjectEntityRiskChangesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableContent(
                    typing.cast(
                        UnprocessableContentResponse,
                        parse_obj_as(
                            type_=UnprocessableContentResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_project_risk_changes(
        self, project_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Dismiss risk changes for all project entities in a project.

        Parameters
        ----------
        project_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from sayari import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.project_entity.delete_project_risk_changes(
            project_id="project_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/changes/risk",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_project_entity_risk_changes(
        self, project_id: str, project_entity_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Dismiss risk changes for a single project entity.

        Parameters
        ----------
        project_id : str

        project_entity_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from sayari import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.project_entity.delete_project_entity_risk_changes(
            project_id="project_id",
            project_entity_id="project_entity_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/{jsonable_encoder(project_entity_id)}/changes/risk",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def project_entity_supply_chain(
        self,
        project_id: str,
        project_entity_id: str,
        *,
        product: typing.Optional[typing.Sequence[str]] = None,
        not_product: typing.Optional[typing.Sequence[str]] = None,
        risk: typing.Optional[typing.Sequence[Risk]] = None,
        not_risk: typing.Optional[typing.Sequence[Risk]] = None,
        countries: typing.Optional[typing.Sequence[Country]] = None,
        not_countries: typing.Optional[typing.Sequence[Country]] = None,
        shipment_country: typing.Optional[typing.Sequence[Country]] = None,
        not_shipment_country: typing.Optional[typing.Sequence[Country]] = None,
        tier_1_shipment_country: typing.Optional[typing.Sequence[Country]] = None,
        tier_2_shipment_country: typing.Optional[typing.Sequence[Country]] = None,
        tier_3_shipment_country: typing.Optional[typing.Sequence[Country]] = None,
        tier_4_shipment_country: typing.Optional[typing.Sequence[Country]] = None,
        tier_5_shipment_country: typing.Optional[typing.Sequence[Country]] = None,
        component: typing.Optional[typing.Sequence[str]] = None,
        not_component: typing.Optional[typing.Sequence[str]] = None,
        min_date: typing.Optional[str] = None,
        max_date: typing.Optional[str] = None,
        max_depth: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpstreamTradeTraversalResponse:
        """
        Execute a traversal of the upstream trade network (supply chain) for all matched entities of a project entity, returning a set of entities and edges between them.

        Parameters
        ----------
        project_id : str
            The project Identifier

        project_entity_id : str
            The project entity Identifier

        product : typing.Optional[typing.Sequence[str]]
            Product root edge filter. Filters results to include only trade relationships where the associated component is part of the specified product's blueprint or is a sub-component of that product.

        not_product : typing.Optional[typing.Sequence[str]]
            Product root edge filter. Filters results to exclude any trade relationships where the associated component is part of the specified product's blueprint or is a sub-component of that product.

        risk : typing.Optional[typing.Sequence[Risk]]
            Risk leaf node filter. Only return supply chains that end with a supplier that has 1+ of the specified risk factors.

        not_risk : typing.Optional[typing.Sequence[Risk]]
            Risk leaf node filter. Only return supply chains that end with a supplier that has none of the specified risk factors.

        countries : typing.Optional[typing.Sequence[Country]]
            Country leaf node filter. Only return supply chains that end with a supplier in 1+ of the specified countries.

        not_countries : typing.Optional[typing.Sequence[Country]]
            Country leaf node filter. Only return supply chains that end with a supplier in none of the specified countries.

        shipment_country : typing.Optional[typing.Sequence[Country]]
            Filters supply chain paths where 1+ shipment country from any tier matches the provided values.

        not_shipment_country : typing.Optional[typing.Sequence[Country]]
            Filters supply chain paths where none of the shipment countries from any tier matches the provided values.

        tier_1_shipment_country : typing.Optional[typing.Sequence[Country]]
            Filters supply chain paths where 1+ shipment country from tier 1 matches the provided values.

        tier_2_shipment_country : typing.Optional[typing.Sequence[Country]]
            Filters supply chain paths where 1+ shipment country from tier 2 matches the provided values.

        tier_3_shipment_country : typing.Optional[typing.Sequence[Country]]
            Filters supply chain paths where 1+ shipment country from tier 3 matches the provided values.

        tier_4_shipment_country : typing.Optional[typing.Sequence[Country]]
            Filters supply chain paths where 1+ shipment country from tier 4 matches the provided values.

        tier_5_shipment_country : typing.Optional[typing.Sequence[Country]]
            Filters supply chain paths where 1+ shipment country from tier 5 matches the provided values.

        component : typing.Optional[typing.Sequence[str]]
            Component edge filter. Only return supply chains that contain at least one edge with 1+ of the specified HS codes.

        not_component : typing.Optional[typing.Sequence[str]]
            Component edge filter. Only return supply chains that contain no edges with any of the specified HS codes.

        min_date : typing.Optional[str]
            Minimum date edge filter in <YYYY-MM-DD> format. Only return supply chains with edge dates that are greater than or equal to this date.

        max_date : typing.Optional[str]
            Maximum date edge filter in <YYYY-MM-DD> format. Only return supply chains with edge dates that are less than or equal to this date.

        max_depth : typing.Optional[int]
            The maximum depth of the traversal, from 1 to 4 inclusive. Default is 4. Reduce if query is timing out.

        limit : typing.Optional[int]
            The maximum number of results to return. Default is no limit.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpstreamTradeTraversalResponse

        Examples
        --------
        from sayari import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.project_entity.project_entity_supply_chain(
            project_id="0n4473",
            project_entity_id="yebNPJ",
            product=["6004"],
            limit=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/{jsonable_encoder(project_entity_id)}/supply_chain/upstream",
            method="GET",
            params={
                "product": product,
                "-product": not_product,
                "risk": risk,
                "-risk": not_risk,
                "countries": countries,
                "-countries": not_countries,
                "shipment_country": shipment_country,
                "-shipment_country": not_shipment_country,
                "tier1_shipment_country": tier_1_shipment_country,
                "tier2_shipment_country": tier_2_shipment_country,
                "tier3_shipment_country": tier_3_shipment_country,
                "tier4_shipment_country": tier_4_shipment_country,
                "tier5_shipment_country": tier_5_shipment_country,
                "component": component,
                "-component": not_component,
                "min_date": min_date,
                "max_date": max_date,
                "max_depth": max_depth,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UpstreamTradeTraversalResponse,
                    parse_obj_as(
                        type_=UpstreamTradeTraversalResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_project_entity_risk_summary(
        self,
        project_id: str,
        project_entity_id: str,
        *,
        filter: ProjectEntityRiskSummaryFilters,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectEntityRiskSummaryResponse:
        """
        Retrieves a risk summary for a specific project entity, including risk factors with network paths and risk intelligence data.

        **Response includes:**
        - Risk factors with their levels (elevated, high, critical)
        - Network paths showing relationships between entities
        - Risk intelligence scores and metadata
        - Risk categories and source entity information

        Parameters
        ----------
        project_id : str

        project_entity_id : str

        filter : ProjectEntityRiskSummaryFilters
            Filter risk factors by risk factor IDs and risk categories

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectEntityRiskSummaryResponse

        Examples
        --------
        from sayari import Sayari
        from sayari.project_entity import ProjectEntityRiskSummaryFilters

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.project_entity.get_project_entity_risk_summary(
            project_id="YVB88Y",
            project_entity_id="52z4Wa",
            filter=ProjectEntityRiskSummaryFilters(
                risk_factor=["sanctioned", "regulatory_action"],
                risk_category=["sanctions", "export_controls"],
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/{jsonable_encoder(project_entity_id)}/risk_summary",
            method="GET",
            params={
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=ProjectEntityRiskSummaryFilters, direction="write"
                ),
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ProjectEntityRiskSummaryResponse,
                    parse_obj_as(
                        type_=ProjectEntityRiskSummaryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def project_entity_supply_chain_summary(
        self,
        project_id: str,
        project_entity_id: str,
        *,
        product: typing.Optional[typing.Sequence[str]] = None,
        not_product: typing.Optional[typing.Sequence[str]] = None,
        risk_factors: typing.Optional[typing.Sequence[Risk]] = None,
        not_risk: typing.Optional[typing.Sequence[Risk]] = None,
        countries: typing.Optional[typing.Sequence[Country]] = None,
        not_countries: typing.Optional[typing.Sequence[Country]] = None,
        component: typing.Optional[typing.Sequence[str]] = None,
        not_component: typing.Optional[typing.Sequence[str]] = None,
        min_date: typing.Optional[str] = None,
        max_date: typing.Optional[str] = None,
        max_depth: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectEntitySupplyChainSummaryResponse:
        """
        Execute a traversal of the upstream trade network (supply chain) for all matched entities of a project entity, returning a summary of countries, risks, and components.

        Parameters
        ----------
        project_id : str
            The project Identifier

        project_entity_id : str
            The project entity Identifier

        product : typing.Optional[typing.Sequence[str]]
            Product root edge filter. Filters results to include only trade relationships where the associated component is part of the specified product's blueprint or is a sub-component of that product.

        not_product : typing.Optional[typing.Sequence[str]]
            Product root edge filter. Filters results to exclude any trade relationships where the associated component is part of the specified product's blueprint or is a sub-component of that product.

        risk_factors : typing.Optional[typing.Sequence[Risk]]
            Risk leaf node filter. Only return supply chains that end with a supplier that has 1+ of the specified risk factors.

        not_risk : typing.Optional[typing.Sequence[Risk]]
            Risk leaf node filter. Only return supply chains that end with a supplier that has none of the specified risk factors.

        countries : typing.Optional[typing.Sequence[Country]]
            Country leaf node filter. Only return supply chains that end with a supplier in 1+ of the specified countries.

        not_countries : typing.Optional[typing.Sequence[Country]]
            Country leaf node filter. Only return supply chains that end with a supplier in none of the specified countries.

        component : typing.Optional[typing.Sequence[str]]
            Component edge filter. Only return supply chains that contain at least one edge with 1+ of the specified HS codes.

        not_component : typing.Optional[typing.Sequence[str]]
            Component edge filter. Only return supply chains that contain no edges with any of the specified HS codes.

        min_date : typing.Optional[str]
            Minimum date edge filter in <YYYY-MM-DD> format. Only return supply chains with edge dates that are greater than or equal to this date.

        max_date : typing.Optional[str]
            Maximum date edge filter in <YYYY-MM-DD> format. Only return supply chains with edge dates that are less than or equal to this date.

        max_depth : typing.Optional[int]
            The maximum depth of the traversal, from 1 to 4 inclusive. Default is 4. Reduce if query is timing out.

        limit : typing.Optional[int]
            The maximum number of results to return. Default is no limit.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectEntitySupplyChainSummaryResponse

        Examples
        --------
        from sayari import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.project_entity.project_entity_supply_chain_summary(
            project_id="0n4473",
            project_entity_id="yebNPJ",
            max_depth=4,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/{jsonable_encoder(project_entity_id)}/supply_chain/upstream_summary",
            method="GET",
            params={
                "product": product,
                "-product": not_product,
                "risk_factors": risk_factors,
                "-risk_factors": not_risk,
                "countries": countries,
                "-countries": not_countries,
                "component": component,
                "-component": not_component,
                "min_date": min_date,
                "max_date": max_date,
                "max_depth": max_depth,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ProjectEntitySupplyChainSummaryResponse,
                    parse_obj_as(
                        type_=ProjectEntitySupplyChainSummaryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncProjectEntityClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create_project_entity(
        self,
        project_id: str,
        *,
        request: CreateResolvedProjectEntityRequest,
        enable_llm_clean: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SingleProjectEntityResponse:
        """
        The resolution endpoints allow users to search for matching entities against a provided list of attributes. The endpoint is similar to the search endpoint, except it's tuned to only return the best match so the client doesn't need to do as much or any post-processing work to filter down results.

        Parameters
        ----------
        project_id : str

        request : CreateResolvedProjectEntityRequest

        enable_llm_clean : typing.Optional[bool]
            Whether to enable LLM-based data cleaning to remove noise and standardize entity attributes. Defaults to true if not supplied. Set to false to disable LLM cleaning.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SingleProjectEntityResponse

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari
        from sayari.project_entity import CreateResolvedProjectEntityRequest

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.project_entity.create_project_entity(
                project_id="YVB88Y",
                request=CreateResolvedProjectEntityRequest(
                    name=["VTB Bank"],
                    country=["RUS"],
                    address=["Moscow"],
                    identifier=["253400V1H6ART1UQ0N98"],
                    profile="corporate",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/create",
            method="POST",
            params={
                "enable_llm_clean": enable_llm_clean,
            },
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=CreateResolvedProjectEntityRequest, direction="write"
            ),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    SingleProjectEntityResponse,
                    parse_obj_as(
                        type_=SingleProjectEntityResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_project_entities(
        self,
        project_id: str,
        *,
        next: typing.Optional[str] = None,
        prev: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        filter: typing.Optional[ProjectEntitiesFilter] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectEntitiesResponse:
        """
        Retrieves a list of entities for a specific project with pagination support.

        **Response Formats:**
        - **JSON** (default): Returns structured data with nested objects
        - **CSV**: Returns tabular data with dynamic columns for attributes and risk categories

        **CSV Format:**
        The CSV response includes dynamic columns based on the data:
        - `attribute_{field_name}`: Dynamic columns for each attribute field found in the data
        - `risk_category_{category_id}`: Dynamic columns for each risk category found in the data
        - Standard columns: project_id, project_entity_id, label, project_entity_url, upload_ids, strength, countries, tags, case_status, created_at, match_count, upstream_products, upstream_risk_factors, upstream_countries

        Use the `Accept: text/csv` header to request CSV format.

        Parameters
        ----------
        project_id : str

        next : typing.Optional[str]
            The pagination token for the next page of projects.

        prev : typing.Optional[str]
            The pagination token for the previous page of projects.

        limit : typing.Optional[int]
            Limit total values returned for projects. Defaults to 100. Max 100.

        filter : typing.Optional[ProjectEntitiesFilter]
            Filter the project entities. Supports both dot notation (e.g., 'filter.attribute.name') and bracket notation (e.g., 'filter[attribute][name]') for nested field filtering.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectEntitiesResponse

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.project_entity.get_project_entities(
                project_id="YVB88Y",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities",
            method="GET",
            params={
                "next": next,
                "prev": prev,
                "limit": limit,
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=ProjectEntitiesFilter, direction="write"
                ),
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ProjectEntitiesResponse,
                    parse_obj_as(
                        type_=ProjectEntitiesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_project_entity(
        self, project_id: str, project_entity_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SingleProjectEntityResponse:
        """
        Retrieves a specific entity in a project.

        Parameters
        ----------
        project_id : str

        project_entity_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SingleProjectEntityResponse

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.project_entity.get_project_entity(
                project_id="project_id",
                project_entity_id="project_entity_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/{jsonable_encoder(project_entity_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    SingleProjectEntityResponse,
                    parse_obj_as(
                        type_=SingleProjectEntityResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_project_entity(
        self, project_id: str, project_entity_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Deletes a project entity.

        Parameters
        ----------
        project_id : str

        project_entity_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.project_entity.delete_project_entity(
                project_id="project_id",
                project_entity_id="project_entity_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/{jsonable_encoder(project_entity_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_project_entity_match(
        self,
        project_id: str,
        project_entity_id: str,
        match_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Deletes a specific project entity match.

        Parameters
        ----------
        project_id : str

        project_entity_id : str

        match_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.project_entity.delete_project_entity_match(
                project_id="project_id",
                project_entity_id="project_entity_id",
                match_id="match_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/{jsonable_encoder(project_entity_id)}/matches/{jsonable_encoder(match_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def project_entity_exists(
        self, project_id: str, *, request: ResolutionAttributes, request_options: typing.Optional[RequestOptions] = None
    ) -> ProjectEntityIdResponse:
        """
        Checks if a project entity with the given attributes already exists.

        Parameters
        ----------
        project_id : str

        request : ResolutionAttributes

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectEntityIdResponse

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari
        from sayari.project_entity import ResolutionAttributes

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.project_entity.project_entity_exists(
                project_id="YVB88Y",
                request=ResolutionAttributes(
                    name=["Example Company"],
                    country=["USA"],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/exists",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=ResolutionAttributes, direction="write"
            ),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ProjectEntityIdResponse,
                    parse_obj_as(
                        type_=ProjectEntityIdResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def save_project_entity(
        self,
        project_id: str,
        *,
        request: SaveProjectEntityBody,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectEntityIdResponse:
        """
        Stores a project entity given entity IDs and a list of attributes.

        Parameters
        ----------
        project_id : str

        request : SaveProjectEntityBody

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectEntityIdResponse

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari
        from sayari.project_entity import ResolutionAttributes, SaveProjectEntityBody

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.project_entity.save_project_entity(
                project_id="YVB88Y",
                request=SaveProjectEntityBody(
                    entity_ids=["entity_id_1", "entity_id_2"],
                    attributes=ResolutionAttributes(
                        name=["Example Company"],
                        country=["USA"],
                    ),
                    profile="corporate",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/save",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=SaveProjectEntityBody, direction="write"
            ),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ProjectEntityIdResponse,
                    parse_obj_as(
                        type_=ProjectEntityIdResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_project_risk_changes(
        self,
        project_id: str,
        *,
        limit: typing.Optional[int] = None,
        next: typing.Optional[str] = None,
        prev: typing.Optional[str] = None,
        sort_date: typing.Optional[SortOrder] = None,
        filter_from: typing.Optional[str] = None,
        filter_to: typing.Optional[str] = None,
        filter_risk_factor: typing.Optional[typing.Sequence[str]] = None,
        filter_risk_category: typing.Optional[typing.Sequence[str]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectRiskChangesResponse:
        """
        Retrieves the risk changes for all project entities in a project.

        Parameters
        ----------
        project_id : str

        limit : typing.Optional[int]
            The maximum number of risk changes to return. Default is 100.

        next : typing.Optional[str]
            Token to retrieve the next page of risk changes

        prev : typing.Optional[str]
            Token to retrieve the previous page of risk changes

        sort_date : typing.Optional[SortOrder]
            Sort by timestamp in ascending or descending order. Default is desc.

        filter_from : typing.Optional[str]
            Filter risk changes from the provided date

        filter_to : typing.Optional[str]
            Filter risk changes up to the provided date

        filter_risk_factor : typing.Optional[typing.Sequence[str]]
            Filter risk changes for the provided risk factors

        filter_risk_category : typing.Optional[typing.Sequence[str]]
            Filter risk changes for risk factors from the provided risk categories

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectRiskChangesResponse

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.project_entity.get_project_risk_changes(
                project_id="YVB88Y",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/changes/risk",
            method="GET",
            params={
                "limit": limit,
                "next": next,
                "prev": prev,
                "sort.date": sort_date,
                "filter.from": filter_from,
                "filter.to": filter_to,
                "filter.risk_factor": filter_risk_factor,
                "filter.risk_category": filter_risk_category,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ProjectRiskChangesResponse,
                    parse_obj_as(
                        type_=ProjectRiskChangesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableContent(
                    typing.cast(
                        UnprocessableContentResponse,
                        parse_obj_as(
                            type_=UnprocessableContentResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_project_entity_risk_changes(
        self,
        project_id: str,
        project_entity_id: str,
        *,
        limit: typing.Optional[int] = None,
        next: typing.Optional[str] = None,
        prev: typing.Optional[str] = None,
        sort_date: typing.Optional[SortOrder] = None,
        filter_from: typing.Optional[str] = None,
        filter_to: typing.Optional[str] = None,
        filter_risk_factor: typing.Optional[typing.Sequence[str]] = None,
        filter_risk_category: typing.Optional[typing.Sequence[str]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectEntityRiskChangesResponse:
        """
        Retrieves the risk changes for a single project entity.

        Parameters
        ----------
        project_id : str

        project_entity_id : str

        limit : typing.Optional[int]
            The maximum number of risk changes to return. Default is 100.

        next : typing.Optional[str]
            Token to retrieve the next page of risk changes

        prev : typing.Optional[str]
            Token to retrieve the previous page of risk changes

        sort_date : typing.Optional[SortOrder]
            Sort by timestamp in ascending or descending order. Default is desc.

        filter_from : typing.Optional[str]
            Filter risk changes from the provided date

        filter_to : typing.Optional[str]
            Filter risk changes up to the provided date

        filter_risk_factor : typing.Optional[typing.Sequence[str]]
            Filter risk changes for the provided risk factors

        filter_risk_category : typing.Optional[typing.Sequence[str]]
            Filter risk changes for risk factors from the provided risk categories

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectEntityRiskChangesResponse

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.project_entity.get_project_entity_risk_changes(
                project_id="YVB88Y",
                project_entity_id="52z4Wa",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/{jsonable_encoder(project_entity_id)}/changes/risk",
            method="GET",
            params={
                "limit": limit,
                "next": next,
                "prev": prev,
                "sort.date": sort_date,
                "filter.from": filter_from,
                "filter.to": filter_to,
                "filter.risk_factor": filter_risk_factor,
                "filter.risk_category": filter_risk_category,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ProjectEntityRiskChangesResponse,
                    parse_obj_as(
                        type_=ProjectEntityRiskChangesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableContent(
                    typing.cast(
                        UnprocessableContentResponse,
                        parse_obj_as(
                            type_=UnprocessableContentResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_project_risk_changes(
        self, project_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Dismiss risk changes for all project entities in a project.

        Parameters
        ----------
        project_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.project_entity.delete_project_risk_changes(
                project_id="project_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/changes/risk",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_project_entity_risk_changes(
        self, project_id: str, project_entity_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Dismiss risk changes for a single project entity.

        Parameters
        ----------
        project_id : str

        project_entity_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.project_entity.delete_project_entity_risk_changes(
                project_id="project_id",
                project_entity_id="project_entity_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/{jsonable_encoder(project_entity_id)}/changes/risk",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def project_entity_supply_chain(
        self,
        project_id: str,
        project_entity_id: str,
        *,
        product: typing.Optional[typing.Sequence[str]] = None,
        not_product: typing.Optional[typing.Sequence[str]] = None,
        risk: typing.Optional[typing.Sequence[Risk]] = None,
        not_risk: typing.Optional[typing.Sequence[Risk]] = None,
        countries: typing.Optional[typing.Sequence[Country]] = None,
        not_countries: typing.Optional[typing.Sequence[Country]] = None,
        shipment_country: typing.Optional[typing.Sequence[Country]] = None,
        not_shipment_country: typing.Optional[typing.Sequence[Country]] = None,
        tier_1_shipment_country: typing.Optional[typing.Sequence[Country]] = None,
        tier_2_shipment_country: typing.Optional[typing.Sequence[Country]] = None,
        tier_3_shipment_country: typing.Optional[typing.Sequence[Country]] = None,
        tier_4_shipment_country: typing.Optional[typing.Sequence[Country]] = None,
        tier_5_shipment_country: typing.Optional[typing.Sequence[Country]] = None,
        component: typing.Optional[typing.Sequence[str]] = None,
        not_component: typing.Optional[typing.Sequence[str]] = None,
        min_date: typing.Optional[str] = None,
        max_date: typing.Optional[str] = None,
        max_depth: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpstreamTradeTraversalResponse:
        """
        Execute a traversal of the upstream trade network (supply chain) for all matched entities of a project entity, returning a set of entities and edges between them.

        Parameters
        ----------
        project_id : str
            The project Identifier

        project_entity_id : str
            The project entity Identifier

        product : typing.Optional[typing.Sequence[str]]
            Product root edge filter. Filters results to include only trade relationships where the associated component is part of the specified product's blueprint or is a sub-component of that product.

        not_product : typing.Optional[typing.Sequence[str]]
            Product root edge filter. Filters results to exclude any trade relationships where the associated component is part of the specified product's blueprint or is a sub-component of that product.

        risk : typing.Optional[typing.Sequence[Risk]]
            Risk leaf node filter. Only return supply chains that end with a supplier that has 1+ of the specified risk factors.

        not_risk : typing.Optional[typing.Sequence[Risk]]
            Risk leaf node filter. Only return supply chains that end with a supplier that has none of the specified risk factors.

        countries : typing.Optional[typing.Sequence[Country]]
            Country leaf node filter. Only return supply chains that end with a supplier in 1+ of the specified countries.

        not_countries : typing.Optional[typing.Sequence[Country]]
            Country leaf node filter. Only return supply chains that end with a supplier in none of the specified countries.

        shipment_country : typing.Optional[typing.Sequence[Country]]
            Filters supply chain paths where 1+ shipment country from any tier matches the provided values.

        not_shipment_country : typing.Optional[typing.Sequence[Country]]
            Filters supply chain paths where none of the shipment countries from any tier matches the provided values.

        tier_1_shipment_country : typing.Optional[typing.Sequence[Country]]
            Filters supply chain paths where 1+ shipment country from tier 1 matches the provided values.

        tier_2_shipment_country : typing.Optional[typing.Sequence[Country]]
            Filters supply chain paths where 1+ shipment country from tier 2 matches the provided values.

        tier_3_shipment_country : typing.Optional[typing.Sequence[Country]]
            Filters supply chain paths where 1+ shipment country from tier 3 matches the provided values.

        tier_4_shipment_country : typing.Optional[typing.Sequence[Country]]
            Filters supply chain paths where 1+ shipment country from tier 4 matches the provided values.

        tier_5_shipment_country : typing.Optional[typing.Sequence[Country]]
            Filters supply chain paths where 1+ shipment country from tier 5 matches the provided values.

        component : typing.Optional[typing.Sequence[str]]
            Component edge filter. Only return supply chains that contain at least one edge with 1+ of the specified HS codes.

        not_component : typing.Optional[typing.Sequence[str]]
            Component edge filter. Only return supply chains that contain no edges with any of the specified HS codes.

        min_date : typing.Optional[str]
            Minimum date edge filter in <YYYY-MM-DD> format. Only return supply chains with edge dates that are greater than or equal to this date.

        max_date : typing.Optional[str]
            Maximum date edge filter in <YYYY-MM-DD> format. Only return supply chains with edge dates that are less than or equal to this date.

        max_depth : typing.Optional[int]
            The maximum depth of the traversal, from 1 to 4 inclusive. Default is 4. Reduce if query is timing out.

        limit : typing.Optional[int]
            The maximum number of results to return. Default is no limit.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpstreamTradeTraversalResponse

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.project_entity.project_entity_supply_chain(
                project_id="0n4473",
                project_entity_id="yebNPJ",
                product=["6004"],
                limit=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/{jsonable_encoder(project_entity_id)}/supply_chain/upstream",
            method="GET",
            params={
                "product": product,
                "-product": not_product,
                "risk": risk,
                "-risk": not_risk,
                "countries": countries,
                "-countries": not_countries,
                "shipment_country": shipment_country,
                "-shipment_country": not_shipment_country,
                "tier1_shipment_country": tier_1_shipment_country,
                "tier2_shipment_country": tier_2_shipment_country,
                "tier3_shipment_country": tier_3_shipment_country,
                "tier4_shipment_country": tier_4_shipment_country,
                "tier5_shipment_country": tier_5_shipment_country,
                "component": component,
                "-component": not_component,
                "min_date": min_date,
                "max_date": max_date,
                "max_depth": max_depth,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UpstreamTradeTraversalResponse,
                    parse_obj_as(
                        type_=UpstreamTradeTraversalResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_project_entity_risk_summary(
        self,
        project_id: str,
        project_entity_id: str,
        *,
        filter: ProjectEntityRiskSummaryFilters,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectEntityRiskSummaryResponse:
        """
        Retrieves a risk summary for a specific project entity, including risk factors with network paths and risk intelligence data.

        **Response includes:**
        - Risk factors with their levels (elevated, high, critical)
        - Network paths showing relationships between entities
        - Risk intelligence scores and metadata
        - Risk categories and source entity information

        Parameters
        ----------
        project_id : str

        project_entity_id : str

        filter : ProjectEntityRiskSummaryFilters
            Filter risk factors by risk factor IDs and risk categories

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectEntityRiskSummaryResponse

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari
        from sayari.project_entity import ProjectEntityRiskSummaryFilters

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.project_entity.get_project_entity_risk_summary(
                project_id="YVB88Y",
                project_entity_id="52z4Wa",
                filter=ProjectEntityRiskSummaryFilters(
                    risk_factor=["sanctioned", "regulatory_action"],
                    risk_category=["sanctions", "export_controls"],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/{jsonable_encoder(project_entity_id)}/risk_summary",
            method="GET",
            params={
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=ProjectEntityRiskSummaryFilters, direction="write"
                ),
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ProjectEntityRiskSummaryResponse,
                    parse_obj_as(
                        type_=ProjectEntityRiskSummaryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def project_entity_supply_chain_summary(
        self,
        project_id: str,
        project_entity_id: str,
        *,
        product: typing.Optional[typing.Sequence[str]] = None,
        not_product: typing.Optional[typing.Sequence[str]] = None,
        risk_factors: typing.Optional[typing.Sequence[Risk]] = None,
        not_risk: typing.Optional[typing.Sequence[Risk]] = None,
        countries: typing.Optional[typing.Sequence[Country]] = None,
        not_countries: typing.Optional[typing.Sequence[Country]] = None,
        component: typing.Optional[typing.Sequence[str]] = None,
        not_component: typing.Optional[typing.Sequence[str]] = None,
        min_date: typing.Optional[str] = None,
        max_date: typing.Optional[str] = None,
        max_depth: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ProjectEntitySupplyChainSummaryResponse:
        """
        Execute a traversal of the upstream trade network (supply chain) for all matched entities of a project entity, returning a summary of countries, risks, and components.

        Parameters
        ----------
        project_id : str
            The project Identifier

        project_entity_id : str
            The project entity Identifier

        product : typing.Optional[typing.Sequence[str]]
            Product root edge filter. Filters results to include only trade relationships where the associated component is part of the specified product's blueprint or is a sub-component of that product.

        not_product : typing.Optional[typing.Sequence[str]]
            Product root edge filter. Filters results to exclude any trade relationships where the associated component is part of the specified product's blueprint or is a sub-component of that product.

        risk_factors : typing.Optional[typing.Sequence[Risk]]
            Risk leaf node filter. Only return supply chains that end with a supplier that has 1+ of the specified risk factors.

        not_risk : typing.Optional[typing.Sequence[Risk]]
            Risk leaf node filter. Only return supply chains that end with a supplier that has none of the specified risk factors.

        countries : typing.Optional[typing.Sequence[Country]]
            Country leaf node filter. Only return supply chains that end with a supplier in 1+ of the specified countries.

        not_countries : typing.Optional[typing.Sequence[Country]]
            Country leaf node filter. Only return supply chains that end with a supplier in none of the specified countries.

        component : typing.Optional[typing.Sequence[str]]
            Component edge filter. Only return supply chains that contain at least one edge with 1+ of the specified HS codes.

        not_component : typing.Optional[typing.Sequence[str]]
            Component edge filter. Only return supply chains that contain no edges with any of the specified HS codes.

        min_date : typing.Optional[str]
            Minimum date edge filter in <YYYY-MM-DD> format. Only return supply chains with edge dates that are greater than or equal to this date.

        max_date : typing.Optional[str]
            Maximum date edge filter in <YYYY-MM-DD> format. Only return supply chains with edge dates that are less than or equal to this date.

        max_depth : typing.Optional[int]
            The maximum depth of the traversal, from 1 to 4 inclusive. Default is 4. Reduce if query is timing out.

        limit : typing.Optional[int]
            The maximum number of results to return. Default is no limit.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ProjectEntitySupplyChainSummaryResponse

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.project_entity.project_entity_supply_chain_summary(
                project_id="0n4473",
                project_entity_id="yebNPJ",
                max_depth=4,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/entities/{jsonable_encoder(project_entity_id)}/supply_chain/upstream_summary",
            method="GET",
            params={
                "product": product,
                "-product": not_product,
                "risk_factors": risk_factors,
                "-risk_factors": not_risk,
                "countries": countries,
                "-countries": not_countries,
                "component": component,
                "-component": not_component,
                "min_date": min_date,
                "max_date": max_date,
                "max_depth": max_depth,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ProjectEntitySupplyChainSummaryResponse,
                    parse_obj_as(
                        type_=ProjectEntitySupplyChainSummaryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .ontology_sources_filters import OntologySourcesFilters
import typing
from .ontology_source import OntologySource
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class GetOntologySourcesResponse(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from sayari.ontology import (
        GetOntologySourcesResponse,
        OntologySource,
        OntologySourcesFilters,
    )

    GetOntologySourcesResponse(
        filters=OntologySourcesFilters(
            id=["e85d865943ee6d8369307569d2ad9de0"],
            country=["XXX"],
            source_type=["adverse_media_/_negative_news_data"],
        ),
        data=[
            OntologySource(
                id="e85d865943ee6d8369307569d2ad9de0",
                label="Acuris Risk Intelligence Adverse Media Data",
                description="Contains PDFs and URLs to adverse media reporting for PEPs, SOEs, sanctioned entities, and entities linked to financial regulatory and law enforcement actions. Available for millions of entities from 'Acuris Risk Intelligence KYC6 (3rd Party Data)' in 'Records' section.",
                country="XXX",
                region="international_(multi-region_coverage)",
                date_added=datetime.date.fromisoformat(
                    "2022-04-11",
                ),
                source_type="adverse_media_/_negative_news_data",
                record_type="adverse_media_record",
                structure="unstructured",
                source_url="https://www.acurisriskintelligence.com/",
                pep=False,
                watchlist=False,
            )
        ],
        total_count=1234,
    )
    """

    filters: OntologySourcesFilters
    data: typing.List[OntologySource]
    total_count: int

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
